self.__BUILD_MANIFEST=function(s,a){return{__rewrites:{beforeFiles:[],afterFiles:[],fallback:[]},"/":["static/chunks/pages/index-d060cd3e3efca4df.js"],"/_error":["static/chunks/pages/_error-674c3a2f7728554d.js"],"/datasets":[s,a,"static/chunks/pages/datasets-3a65952f8a8a3659.js"],"/datasets/[...slug]":["static/chunks/29107295-adc1ad4a30256858.js","static/chunks/8a28b14e-c61a767b3439f57c.js",s,"static/chunks/112-4f9a6a9218683dda.js",a,"static/chunks/pages/datasets/[...slug]-dca317778630c2fc.js"],sortedPages:["/","/_app","/_error","/datasets","/datasets/[...slug]"]}}("static/chunks/303-03c350f9af236a63.js","static/chunks/489-80e7b88537d385a9.js"),self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB();