(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[405],{6282:function(n,u,t){n.exports=t(5575)},3878:function(n,u,t){"use strict";t.r(u);var f=t(6282),c=t.n(f),e=t(2771);u.default=function(){(0,e.useEffect)((function(){c().push("/datasets")}))}},8312:function(n,u,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/",function(){return t(3878)}])}},function(n){n.O(0,[774,888,179],(function(){return u=8312,n(n.s=u);var u}));var u=n.O();_N_E=u}]);