a,abbr,acronym,address,applet,article,aside,audio,b,big,blockquote,body,canvas,caption,center,cite,code,dd,del,details,dfn,div,dl,dt,em,embed,fieldset,figcaption,figure,footer,form,h1,h2,h3,h4,h5,h6,header,hgroup,html,i,iframe,img,ins,kbd,label,legend,li,mark,menu,nav,object,ol,output,p,pre,q,ruby,s,samp,section,small,span,strike,strong,sub,summary,sup,table,tbody,td,tfoot,th,thead,time,tr,tt,u,ul,var,video{border:0;box-sizing:border-box;font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Oxygen,Ubuntu,Cantarell,Fira Sans,Droid Sans,Helvetica Neue,sans-serif;font-size:100%;font:inherit;margin:0;padding:0;vertical-align:baseline}:focus{outline:0}article,aside,details,figcaption,figure,footer,header,hgroup,menu,nav,section{display:block}a{color:inherit;text-decoration:none}body{line-height:1;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;overflow-x:hidden}ol,ul{margin:0}ul{padding-left:24px}ol{padding-left:32px}blockquote,q{quotes:none}blockquote:after,blockquote:before,q:after,q:before{content:"";content:none}input[type=search]::-webkit-search-cancel-button,input[type=search]::-webkit-search-decoration,input[type=search]::-webkit-search-results-button,input[type=search]::-webkit-search-results-decoration{-webkit-appearance:none;-moz-appearance:none}input[type=search]{-webkit-appearance:none;-moz-appearance:none}audio,canvas,video{display:inline-block;*display:inline;*zoom:1;max-width:100%}audio:not([controls]){display:none;height:0}[hidden]{display:none}html{font-size:100%;-webkit-text-size-adjust:100%;-ms-text-size-adjust:100%}a:focus{outline:thin dotted}a:active,a:hover{outline:0}img{border:0;-ms-interpolation-mode:bicubic}figure,form{margin:0}fieldset{border:1px solid silver;margin:0 2px;padding:.35em .625em .75em}legend{border:0;padding:0;white-space:normal;*margin-left:-7px}button,input,select,textarea{font-size:100%;margin:0;vertical-align:baseline;*vertical-align:middle}button,input{line-height:normal}button,select{text-transform:none}button,html input[type=button],input[type=reset],input[type=submit]{-webkit-appearance:button;cursor:pointer;*overflow:visible}button[disabled],html input[disabled],html select[disabled],html select[disabled]:hover{cursor:not-allowed}input[type=checkbox],input[type=radio]{box-sizing:border-box;padding:0;*height:13px;*width:13px}input[type=radio]{-webkit-appearance:none;-moz-appearance:none;appearance:none;display:inline-block;width:16px;height:16px;padding:3px;background-clip:content-box;border:1px solid #b3b3bb;background-color:#f8f8f8;border-radius:50%}input[type=radio]:checked{background-color:#00916e}input[type=search]{-webkit-appearance:textfield;box-sizing:content-box}input[type=search]::-webkit-search-cancel-button,input[type=search]::-webkit-search-decoration{-webkit-appearance:none}button::-moz-focus-inner,input::-moz-focus-inner{border:0;padding:0}textarea{overflow:auto;vertical-align:top}table{border-collapse:collapse;border-spacing:0}button,html,input,select,textarea{color:#222}::-moz-selection{background:#b3d4fc;text-shadow:none}::selection{background:rgba(56,107,233,.25);text-shadow:none}img{vertical-align:middle}fieldset{border:0;margin:0;padding:0}textarea{resize:vertical}.chromeframe{margin:.2em 0;background:#ccc;color:#000;padding:.2em 0}@font-face{font-family:Roboto Thin;font-style:normal;font-weight:100;src:url(/fonts/Roboto/Roboto-Thin.ttf) format("truetype")}@font-face{font-family:Roboto Light;font-style:normal;font-weight:300;src:url(/fonts/Roboto/Roboto-Light.ttf) format("truetype")}@font-face{font-family:Roboto Regular;font-style:normal;font-weight:400;src:url(/fonts/Roboto/Roboto-Regular.ttf) format("truetype")}@font-face{font-family:Roboto Medium;font-style:normal;font-weight:500;src:url(/fonts/Roboto/Roboto-Medium.ttf) format("truetype")}@font-face{font-family:Roboto Bold;font-style:normal;font-weight:700;src:url(/fonts/Roboto/Roboto-Bold.ttf) format("truetype")}@font-face{font-family:Roboto Mono Regular;font-style:normal;font-weight:400;src:url(/fonts/Roboto_Mono/RobotoMono-VariableFont_wght.ttf) format("truetype")}@font-face{font-family:Roboto Mono Bold;font-style:normal;font-weight:700;src:url(/fonts/Roboto_Mono/RobotoMono-VariableFont_wght.ttf) format("truetype")}